<?php

namespace Drupal\static_suite\Command;

use Drupal\static_suite\Command\Result\CliCommandResultFactoryInterface;
use Drupal\static_suite\Command\Result\CliCommandResultInterface;
use Drupal\static_suite\StaticSuiteException;

/**
 * A class for CLI commands.
 */
class CliCommand implements CliCommandInterface {

  /**
   * The CLI command result factory.
   *
   * @var \Drupal\static_suite\Command\Result\CliCommandResultFactoryInterface
   */
  protected $cliCommandResultFactory;

  /**
   * The CLI command to execute.
   *
   * @var string
   */
  protected $cmd;

  /**
   * The working directory in which the call will be executed.
   *
   * @var string|null
   */
  protected $cwd;

  /**
   * Environment variables.
   *
   * @var array|null
   */
  protected $env;

  /**
   * Creates a new instance of a CLI command.
   *
   * @param \Drupal\static_suite\Command\Result\CliCommandResultFactoryInterface $cliCommandResultFactory
   *   The CLI command result factory.
   * @param string $cmd
   *   The CLI command to execute.
   * @param string|null $cwd
   *   The working directory in which the call will be executed.
   * @param array|null $env
   *   Environment variables.
   */
  public function __construct(CliCommandResultFactoryInterface $cliCommandResultFactory, string $cmd, string $cwd = NULL, array $env = NULL) {
    $this->cliCommandResultFactory = $cliCommandResultFactory;
    $this->cmd = $cmd;
    $this->cwd = $cwd;
    $this->env = $env;
  }

  /**
   * {@inheritdoc}
   */
  public function getCmd(): string {
    return $this->cmd;
  }

  /**
   * {@inheritdoc}
   */
  public function getCwd(): ?string {
    return $this->cwd;
  }

  /**
   * {@inheritdoc}
   */
  public function getEnv(): ?array {
    return $this->env;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(string $stdIn = NULL): CliCommandResultInterface {
    $descriptorSpec = [
      0 => ["pipe", "r"],
      1 => ["pipe", "w"],
      2 => ["pipe", "w"],
    ];
    $pipes = [];
    $process = proc_open(
      $this->getCmd(),
      $descriptorSpec,
      $pipes,
      $this->getCwd(),
      $this->getEnv()
    );

    if (is_resource($process)) {
      if ($stdIn !== NULL) {
        fwrite($pipes[0], $stdIn);
      }
      fclose($pipes[0]);

      $stdOutContents = stream_get_contents($pipes[1]);
      fclose($pipes[1]);

      $stdErrContents = stream_get_contents($pipes[2]);
      fclose($pipes[2]);

      $returnCode = proc_close($process);

      return $this->cliCommandResultFactory->create($returnCode, $stdOutContents, $stdErrContents);
    }

    fclose($pipes[1]);
    fclose($pipes[2]);
    throw new StaticSuiteException(sprintf('Cannot execute "%s"', $this->getCmd()));
  }

}
